@@ -19,34 +19,85 @@
 
 
 
-GString __thread *crypto_debug_string;
+__thread GString *crypto_debug_string;
 
 
 
-static int aes_cm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, u_int64_t);
-static int aes_cm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, u_int64_t);
-static int hmac_sha1_rtp(struct crypto_context *, char *out, str *in, u_int64_t);
+static int aes_cm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint64_t);
+static int aes_cm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint64_t);
+static int aes_gcm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint64_t);
+static int aes_gcm_decrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint64_t);
+static int aes_gcm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint64_t);
+static int aes_gcm_decrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint64_t);
+static int hmac_sha1_rtp(struct crypto_context *, char *out, str *in, uint64_t);
 static int hmac_sha1_rtcp(struct crypto_context *, char *out, str *in);
-static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, u_int64_t idx);
-static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, u_int64_t idx);
+static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx);
+static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx);
 static int aes_cm_session_key_init(struct crypto_context *c);
+static int aes_gcm_session_key_init(struct crypto_context *c);
 static int aes_f8_session_key_init(struct crypto_context *c);
 static int evp_session_key_cleanup(struct crypto_context *c);
-static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, u_int64_t idx);
-static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, u_int64_t idx);
+static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx);
+static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx);
 
 /* all lengths are in bytes */
 struct crypto_suite __crypto_suites[] = {
 	{
-		.name			= "AES_CM_128_HMAC_SHA1_80",
-		.dtls_name		= "SRTP_AES128_CM_SHA1_80",
+		.name			= "AEAD_AES_256_GCM",
+		.dtls_name		= "SRTP_AEAD_AES_256_GCM",
+		.master_key_len		= 32,
+		.master_salt_len	= 12,
+		.session_key_len	= 32,
+		.session_salt_len	= 12,
+		.srtp_lifetime		= 1ULL << 48,
+		.srtcp_lifetime		= 1ULL << 31,
+		.kernel_cipher		= REC_AEAD_AES_GCM_256,
+		.kernel_hmac		= REH_NULL,
+		.srtp_auth_tag		= 0,
+		.srtcp_auth_tag		= 0,
+		.srtp_auth_key_len	= 0,
+		.srtcp_auth_key_len	= 0,
+		.encrypt_rtp		= aes_gcm_encrypt_rtp,
+		.decrypt_rtp		= aes_gcm_decrypt_rtp,
+		.encrypt_rtcp		= aes_gcm_encrypt_rtcp,
+		.decrypt_rtcp		= aes_gcm_decrypt_rtcp,
+		.session_key_init	= aes_gcm_session_key_init,
+		.session_key_cleanup	= evp_session_key_cleanup,
+		.aead_evp		= EVP_aes_256_gcm,
+	},
+	{
+		.name			= "AEAD_AES_128_GCM",
+		.dtls_name		= "SRTP_AEAD_AES_128_GCM",
 		.master_key_len		= 16,
-		.master_salt_len	= 14,
+		.master_salt_len	= 12,
 		.session_key_len	= 16,
+		.session_salt_len	= 12,
+		.srtp_lifetime		= 1ULL << 48,
+		.srtcp_lifetime		= 1ULL << 31,
+		.kernel_cipher		= REC_AEAD_AES_GCM_128,
+		.kernel_hmac		= REH_NULL,
+		.srtp_auth_tag		= 0,
+		.srtcp_auth_tag		= 0,
+		.srtp_auth_key_len	= 0,
+		.srtcp_auth_key_len	= 0,
+		.encrypt_rtp		= aes_gcm_encrypt_rtp,
+		.decrypt_rtp		= aes_gcm_decrypt_rtp,
+		.encrypt_rtcp		= aes_gcm_encrypt_rtcp,
+		.decrypt_rtcp		= aes_gcm_decrypt_rtcp,
+		.session_key_init	= aes_gcm_session_key_init,
+		.session_key_cleanup	= evp_session_key_cleanup,
+		.aead_evp		= EVP_aes_128_gcm,
+	},
+	{
+		.name			= "AES_256_CM_HMAC_SHA1_80",
+		//.dtls_name		= "SRTP_AES128_CM_SHA1_80",
+		.master_key_len		= 32,
+		.master_salt_len	= 14,
+		.session_key_len	= 32,
 		.session_salt_len	= 14,
 		.srtp_lifetime		= 1ULL << 48,
 		.srtcp_lifetime		= 1ULL << 31,
-		.kernel_cipher		= REC_AES_CM_128,
+		.kernel_cipher		= REC_AES_CM_256,
 		.kernel_hmac		= REH_HMAC_SHA1,
 		.srtp_auth_tag		= 10,
 		.srtcp_auth_tag		= 10,
@@ -62,15 +113,15 @@ struct crypto_suite __crypto_suites[] = {
 		.session_key_cleanup	= evp_session_key_cleanup,
 	},
 	{
-		.name			= "AES_CM_128_HMAC_SHA1_32",
-		.dtls_name		= "SRTP_AES128_CM_SHA1_32",
-		.master_key_len		= 16,
+		.name			= "AES_256_CM_HMAC_SHA1_32",
+		//.dtls_name		= "SRTP_AES128_CM_SHA1_32",
+		.master_key_len		= 32,
 		.master_salt_len	= 14,
-		.session_key_len	= 16,
+		.session_key_len	= 32,
 		.session_salt_len	= 14,
 		.srtp_lifetime		= 1ULL << 48,
 		.srtcp_lifetime		= 1ULL << 31,
-		.kernel_cipher		= REC_AES_CM_128,
+		.kernel_cipher		= REC_AES_CM_256,
 		.kernel_hmac		= REH_HMAC_SHA1,
 		.srtp_auth_tag		= 4,
 		.srtcp_auth_tag		= 10,
@@ -134,15 +185,15 @@ struct crypto_suite __crypto_suites[] = {
 		.session_key_cleanup	= evp_session_key_cleanup,
 	},
 	{
-		.name			= "AES_256_CM_HMAC_SHA1_80",
-		//.dtls_name		= "SRTP_AES128_CM_SHA1_80",
-		.master_key_len		= 32,
+		.name			= "AES_CM_128_HMAC_SHA1_80",
+		.dtls_name		= "SRTP_AES128_CM_SHA1_80",
+		.master_key_len		= 16,
 		.master_salt_len	= 14,
-		.session_key_len	= 32,
+		.session_key_len	= 16,
 		.session_salt_len	= 14,
 		.srtp_lifetime		= 1ULL << 48,
 		.srtcp_lifetime		= 1ULL << 31,
-		.kernel_cipher		= REC_AES_CM_256,
+		.kernel_cipher		= REC_AES_CM_128,
 		.kernel_hmac		= REH_HMAC_SHA1,
 		.srtp_auth_tag		= 10,
 		.srtcp_auth_tag		= 10,
@@ -158,15 +209,15 @@ struct crypto_suite __crypto_suites[] = {
 		.session_key_cleanup	= evp_session_key_cleanup,
 	},
 	{
-		.name			= "AES_256_CM_HMAC_SHA1_32",
-		//.dtls_name		= "SRTP_AES128_CM_SHA1_32",
-		.master_key_len		= 32,
+		.name			= "AES_CM_128_HMAC_SHA1_32",
+		.dtls_name		= "SRTP_AES128_CM_SHA1_32",
+		.master_key_len		= 16,
 		.master_salt_len	= 14,
-		.session_key_len	= 32,
+		.session_key_len	= 16,
 		.session_salt_len	= 14,
 		.srtp_lifetime		= 1ULL << 48,
 		.srtcp_lifetime		= 1ULL << 31,
-		.kernel_cipher		= REC_AES_CM_256,
+		.kernel_cipher		= REC_AES_CM_128,
 		.kernel_hmac		= REH_HMAC_SHA1,
 		.srtp_auth_tag		= 4,
 		.srtcp_auth_tag		= 10,
@@ -315,7 +366,8 @@ static void aes_ctr(unsigned char *out, str *in, EVP_CIPHER_CTX *ecc, const unsi
 	unsigned char *p, *q;
 	unsigned int left;
 	int outlen, i;
-	u_int64_t *pi, *qi, *ki;
+	gboolean aligned = TRUE;
+	uint64_t *pi, *qi, *ki;
 
 	if (!ecc)
 		return;
@@ -326,11 +378,16 @@ static void aes_ctr(unsigned char *out, str *in, EVP_CIPHER_CTX *ecc, const unsi
 	ki = (void *) key_block;
 	left = in->len;
 
+	if ((GPOINTER_TO_UINT(pi) % sizeof(*pi)) != 0)
+		aligned = FALSE;
+	if ((GPOINTER_TO_UINT(qi) % sizeof(*qi)) != 0)
+		aligned = FALSE;
+
 	while (left) {
 		EVP_EncryptUpdate(ecc, key_block, &outlen, ivx, 16);
 		assert(outlen == 16);
 
-		if (G_UNLIKELY(left < 16)) {
+		if (left < 16 || !aligned) {
 			p = (void *) pi;
 			q = (void *) qi;
 			for (i = 0; i < 16; i++) {
@@ -339,12 +396,14 @@ static void aes_ctr(unsigned char *out, str *in, EVP_CIPHER_CTX *ecc, const unsi
 				if (!left)
 					goto done;
 			}
-			abort();
 		}
-
-		*qi++ = *pi++ ^ ki[0];
-		*qi++ = *pi++ ^ ki[1];
-		left -= 16;
+		else {
+			qi[0] = pi[0] ^ ki[0];
+			qi[1] = pi[1] ^ ki[1];
+			left -= 16;
+		}
+		qi += 2;
+		pi += 2;
 
 		for (i = 15; i >= 0; i--) {
 			ivx[i]++;
@@ -413,37 +472,44 @@ int crypto_gen_session_key(struct crypto_context *c, str *out, unsigned char lab
 	unsigned char x[14];
 	int i;
 
+	if (!out->len)
+		return 0;
+
 	ZERO(key_id);
 	/* key_id[1..6] := r; or 1..4 for rtcp
 	 * key_derivation_rate == 0 --> r == 0 */
 
 	key_id[0] = label;
-	memcpy(x, c->params.master_salt, 14);
+	assert(sizeof(x) >= c->params.crypto_suite->master_salt_len);
+	memcpy(x, c->params.master_salt, c->params.crypto_suite->master_salt_len);
+	// AEAD uses 12 bytes master salt; pad on the right to get 14
+	// Errata: https://www.rfc-editor.org/errata_search.php?rfc=7714
+	if (c->params.crypto_suite->master_salt_len == 12)
+		x[12] = x[13] = '\x00';
 	for (i = 13 - index_len; i < 14; i++)
 		x[i] = key_id[i - (13 - index_len)] ^ x[i];
 
-	prf_n(out, c->params.master_key, c->params.crypto_suite->lib_cipher_ptr, x);
+	prf_n(out, c->params.master_key, c->params.crypto_suite->aes_evp, x);
 
-	if (rtpe_config.debug_srtp)
-		ilog(LOG_DEBUG, "Generated session key: master key "
-				"%02x%02x%02x%02x..., "
-				"master salt "
-				"%02x%02x%02x%02x..., "
-				"label %02x, length %i, result "
-				"%02x%02x%02x%02x...",
-				c->params.master_key[0],
-				c->params.master_key[1],
-				c->params.master_key[2],
-				c->params.master_key[3],
-				c->params.master_salt[0],
-				c->params.master_salt[1],
-				c->params.master_salt[2],
-				c->params.master_salt[3],
-				label, out->len,
-				(unsigned char) out->s[0],
-				(unsigned char) out->s[1],
-				(unsigned char) out->s[2],
-				(unsigned char) out->s[3]);
+	ilogs(srtp, LOG_DEBUG, "Generated session key: master key "
+			"%02x%02x%02x%02x..., "
+			"master salt "
+			"%02x%02x%02x%02x..., "
+			"label %02x, length %zu, result "
+			"%02x%02x%02x%02x...",
+			c->params.master_key[0],
+			c->params.master_key[1],
+			c->params.master_key[2],
+			c->params.master_key[3],
+			c->params.master_salt[0],
+			c->params.master_salt[1],
+			c->params.master_salt[2],
+			c->params.master_salt[3],
+			label, out->len,
+			(unsigned char) out->s[0],
+			(unsigned char) out->s[1],
+			(unsigned char) out->s[2],
+			(unsigned char) out->s[3]);
 
 	return 0;
 }
@@ -461,10 +527,10 @@ int crypto_gen_session_key(struct crypto_context *c, str *out, unsigned char lab
  */
 
 /* rfc 3711 section 4.1.1 */
-static int aes_cm_encrypt(struct crypto_context *c, u_int32_t ssrc, str *s, u_int64_t idx) {
+static int aes_cm_encrypt(struct crypto_context *c, uint32_t ssrc, str *s, uint64_t idx) {
 	unsigned char iv[16];
-	u_int32_t *ivi;
-	u_int32_t idxh, idxl;
+	uint32_t *ivi;
+	uint32_t idxh, idxl;
 
 	memcpy(iv, c->session_salt, 14);
 	iv[14] = iv[15] = '\0';
@@ -483,15 +549,168 @@ static int aes_cm_encrypt(struct crypto_context *c, u_int32_t ssrc, str *s, u_in
 }
 
 /* rfc 3711 section 4.1 */
-static int aes_cm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, u_int64_t idx) {
+static int aes_cm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
 	return aes_cm_encrypt(c, r->ssrc, s, idx);
 }
 
 /* rfc 3711 sections 3.4 and 4.1 */
-static int aes_cm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, u_int64_t idx) {
+static int aes_cm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
 	return aes_cm_encrypt(c, r->ssrc, s, idx);
 }
 
+/* rfc 7714 section 8 */
+
+union aes_gcm_rtp_iv {
+	unsigned char bytes[12];
+	struct {
+		uint16_t zeros;
+		uint32_t ssrc;
+		uint32_t roq;
+		uint16_t seq;
+	} __attribute__((__packed__));
+} __attribute__((__packed__));
+
+_Static_assert(offsetof(union aes_gcm_rtp_iv, seq) == 10,
+               "union aes_gcm_rtp_iv not packed");
+
+static int aes_gcm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+	union aes_gcm_rtp_iv iv;
+	int len, ciphertext_len;
+
+	memcpy(iv.bytes, c->session_salt, 12);
+
+	iv.ssrc ^= r->ssrc;
+	iv.roq ^= htonl((idx & 0x00ffffffff0000ULL) >> 16);
+	iv.seq ^= htons(idx & 0x00ffffULL);
+
+	EVP_EncryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aead_evp(), NULL,
+			(const unsigned char *) c->session_key, iv.bytes);
+
+	// nominally 12 bytes of AAD
+	EVP_EncryptUpdate(c->session_key_ctx[0], NULL, &len, (void *)r, s->s - (char *)r);
+
+	EVP_EncryptUpdate(c->session_key_ctx[0], (unsigned char *) s->s, &len,
+			(const unsigned char *) s->s, s->len);
+	ciphertext_len = len;
+	if (!EVP_EncryptFinal_ex(c->session_key_ctx[0], (unsigned char *) s->s+len, &len))
+		return 1;
+	ciphertext_len += len;
+	// append the tag to the str buffer
+	EVP_CIPHER_CTX_ctrl(c->session_key_ctx[0], EVP_CTRL_GCM_GET_TAG, 16, s->s+ciphertext_len);
+	s->len = ciphertext_len + 16;
+
+	return 0;
+}
+
+static int aes_gcm_decrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+	union aes_gcm_rtp_iv iv;
+	int len, plaintext_len;
+
+	if (s->len < 16)
+		return -1;
+
+	memcpy(iv.bytes, c->session_salt, 12);
+
+	iv.ssrc ^= r->ssrc;
+	iv.roq ^= htonl((idx & 0x00ffffffff0000ULL) >> 16);
+	iv.seq ^= htons(idx & 0x00ffffULL);
+
+	EVP_DecryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aead_evp(), NULL,
+			(const unsigned char *) c->session_key, iv.bytes);
+
+	// nominally 12 bytes of AAD
+	EVP_DecryptUpdate(c->session_key_ctx[0], NULL, &len, (void *)r, s->s - (char *)r);
+
+	// decrypt partial buffer - the last 16 bytes are the tag
+	EVP_DecryptUpdate(c->session_key_ctx[0], (unsigned char *) s->s, &len,
+			(const unsigned char *) s->s, s->len-16);
+	plaintext_len = len;
+	EVP_CIPHER_CTX_ctrl(c->session_key_ctx[0], EVP_CTRL_GCM_SET_TAG, 16, s->s + s->len-16);
+	if (!EVP_DecryptFinal_ex(c->session_key_ctx[0], (unsigned char *) s->s+len, &len))
+		return 1;
+	plaintext_len += len;
+	s->len = plaintext_len;
+
+	return 0;
+}
+
+/* rfc 7714 section 9 */
+
+union aes_gcm_rtcp_iv {
+	unsigned char bytes[12];
+	struct {
+		uint16_t zeros_a;
+		uint32_t ssrc;
+		uint16_t zeros_b;
+		uint32_t srtcp;
+	} __attribute__((__packed__));
+} __attribute__((__packed__));
+
+_Static_assert(offsetof(union aes_gcm_rtcp_iv, srtcp) == 8,
+               "union aes_gcm_rtcp_iv not packed");
+
+static int aes_gcm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+	union aes_gcm_rtcp_iv iv;
+	uint32_t e_idx;
+	int len, ciphertext_len;
+
+	memcpy(iv.bytes, c->session_salt, 12);
+
+	iv.ssrc ^= r->ssrc;
+	iv.srtcp ^= htonl(idx & 0x007fffffffULL);
+	e_idx = htonl( (idx&0x007fffffffULL) | 0x80000000);
+
+	EVP_EncryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aead_evp(), NULL,
+			(const unsigned char *) c->session_key, iv.bytes);
+
+	// nominally 8 + 4 bytes of AAD
+	EVP_EncryptUpdate(c->session_key_ctx[0], NULL, &len, (void *)r, s->s - (char *)r);
+	EVP_EncryptUpdate(c->session_key_ctx[0], NULL, &len, (void *)&e_idx, 4);
+
+	EVP_EncryptUpdate(c->session_key_ctx[0], (unsigned char *) s->s, &len,
+			(const unsigned char *) s->s, s->len);
+	ciphertext_len = len;
+	if (!EVP_EncryptFinal_ex(c->session_key_ctx[0], (unsigned char *) s->s+len, &len))
+		return 1;
+	ciphertext_len += len;
+	// append the tag to the str buffer
+	EVP_CIPHER_CTX_ctrl(c->session_key_ctx[0], EVP_CTRL_GCM_GET_TAG, 16, s->s+ciphertext_len);
+	s->len = ciphertext_len + 16;
+
+	return 0;
+}
+
+static int aes_gcm_decrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+	union aes_gcm_rtcp_iv iv;
+	uint32_t e_idx;
+	int len, plaintext_len;
+
+	memcpy(iv.bytes, c->session_salt, 12);
+
+	iv.ssrc ^= r->ssrc;
+	iv.srtcp ^= htonl(idx & 0x007fffffffULL);
+	e_idx = htonl( (idx&0x007fffffffULL) | 0x80000000);
+
+	EVP_DecryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aead_evp(), NULL,
+			(const unsigned char *) c->session_key, iv.bytes);
+
+	// nominally 8 + 4 bytes of AAD
+	EVP_DecryptUpdate(c->session_key_ctx[0], NULL, &len, (void *)r, s->s - (char *)r);
+	EVP_DecryptUpdate(c->session_key_ctx[0], NULL, &len, (void *)&e_idx, 4);
+
+	// decrypt partial buffer - the last 16 bytes are the tag
+	EVP_DecryptUpdate(c->session_key_ctx[0], (unsigned char *) s->s, &len,
+			(const unsigned char *) s->s, s->len-16);
+	plaintext_len = len;
+	EVP_CIPHER_CTX_ctrl(c->session_key_ctx[0], EVP_CTRL_GCM_SET_TAG, 16, s->s + s->len-16);
+	if (!EVP_DecryptFinal_ex(c->session_key_ctx[0], (unsigned char *) s->s+len, &len))
+		return 1;
+	plaintext_len += len;
+	s->len = plaintext_len;
+
+	return 0;
+}
+
 /* rfc 3711 sections 4.1.2 and 4.1.2.1
  * encrypts in place */
 static void aes_128_f8_encrypt(struct crypto_context *c, unsigned char *iv, str *s) {
@@ -499,10 +718,10 @@ static void aes_128_f8_encrypt(struct crypto_context *c, unsigned char *iv, str
 		      ivx[16], /* IV' */
 		      x[16];
 	int i, outlen, left;
-	u_int32_t j;
+	uint32_t j;
 	unsigned char *p;
-	u_int64_t *pi, *ki, *lki, *xi;
-	u_int32_t *xu;
+	uint64_t *pi, *ki, *lki, *xi;
+	uint32_t *xu;
 
 	EVP_EncryptUpdate(c->session_key_ctx[1], ivx, &outlen, iv, 16);
 	assert(outlen == 16);
@@ -554,9 +773,9 @@ done:
 }
 
 /* rfc 3711 section 4.1.2.2 */
-static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, u_int64_t idx) {
+static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
 	unsigned char iv[16];
-	u_int32_t roc;
+	uint32_t roc;
 
 	iv[0] = '\0';
 	memcpy(&iv[1], &r->m_pt, 11); /* m, pt, seq, ts, ssrc */
@@ -569,9 +788,9 @@ static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, st
 }
 
 /* rfc 3711 section 4.1.2.3 */
-static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, u_int64_t idx) {
+static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
 	unsigned char iv[16];
-	u_int32_t i;
+	uint32_t i;
 
 	memset(iv, 0, 4);
 	i = htonl(0x80000000ULL | idx);
@@ -583,9 +802,9 @@ static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r,
 	return 0;
 }
 /* rfc 3711, sections 4.2 and 4.2.1 */
-static int hmac_sha1_rtp(struct crypto_context *c, char *out, str *in, u_int64_t index) {
+static int hmac_sha1_rtp(struct crypto_context *c, char *out, str *in, uint64_t index) {
 	unsigned char hmac[20];
-	u_int32_t roc;
+	uint32_t roc;
 	HMAC_CTX *hc;
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -635,11 +854,23 @@ static int aes_cm_session_key_init(struct crypto_context *c) {
 	c->session_key_ctx[0] = g_slice_alloc(sizeof(EVP_CIPHER_CTX));
 	EVP_CIPHER_CTX_init(c->session_key_ctx[0]);
 #endif
-	EVP_EncryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->lib_cipher_ptr, NULL,
+	EVP_EncryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aes_evp, NULL,
 			(unsigned char *) c->session_key, NULL);
 	return 0;
 }
 
+static int aes_gcm_session_key_init(struct crypto_context *c) {
+	evp_session_key_cleanup(c);
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	c->session_key_ctx[0] = EVP_CIPHER_CTX_new();
+#else
+	c->session_key_ctx[0] = g_slice_alloc(sizeof(EVP_CIPHER_CTX));
+	EVP_CIPHER_CTX_init(c->session_key_ctx[0]);
+#endif
+	return 0;
+}
+
 static int aes_f8_session_key_init(struct crypto_context *c) {
 	unsigned char m[16];
 	int i;
@@ -692,10 +923,10 @@ static int evp_session_key_cleanup(struct crypto_context *c) {
 	return 0;
 }
 
-static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, u_int64_t idx) {
+static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
 	return 0;
 }
-static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, u_int64_t idx) {
+static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
 	return 0;
 }
 
@@ -708,7 +939,7 @@ static void dump_key(struct crypto_context *c, int log_level) {
 	k = g_base64_encode(c->params.master_key, c->params.crypto_suite->master_key_len);
 	s = g_base64_encode(c->params.master_salt, c->params.crypto_suite->master_salt_len);
 
-	ilog(log_level, "--- %s key %s%s%s salt %s%s%s", c->params.crypto_suite->name, FMT_M(k), FMT_M(s));
+	ilogs(srtp, log_level, "--- %s key %s%s%s salt %s%s%s", c->params.crypto_suite->name, FMT_M(k), FMT_M(s));
 
 	g_free(k);
 	g_free(s);
@@ -720,12 +951,12 @@ void crypto_dump_keys(struct crypto_context *in, struct crypto_context *out) {
 	if (rtpe_config.log_keys)
 	    log_level = LOG_ERROR;
 	    
-	if (get_log_level() < log_level)
+	if (get_log_level(core) < log_level)
 		return;
 
-	ilog(log_level, "SRTP keys, incoming:");
+	ilogs(srtp, log_level, "SRTP keys, incoming:");
 	dump_key(in, log_level);
-	ilog(log_level, "SRTP keys, outgoing:");
+	ilogs(srtp, log_level, "SRTP keys, outgoing:");
 	dump_key(out, log_level);
 }
 
@@ -760,13 +991,13 @@ void crypto_init_main() {
 		str_init(&cs->name_str, (char *) cs->name);
 		switch(cs->master_key_len) {
 		case 16:
-			cs->lib_cipher_ptr = EVP_aes_128_ecb();
+			cs->aes_evp = EVP_aes_128_ecb();
 			break;
 		case 24:
-			cs->lib_cipher_ptr = EVP_aes_192_ecb();
+			cs->aes_evp = EVP_aes_192_ecb();
 			break;
 		case 32:
-			cs->lib_cipher_ptr = EVP_aes_256_ecb();
+			cs->aes_evp = EVP_aes_256_ecb();
 			break;
 		}
 	}
