@@ -372,13 +372,17 @@ void on_redis_notification(redisAsyncContext *actx, void *reply, void *privdata)
 		c = call_get(&callid);
 		if (c) {
 			rwlock_unlock_w(&c->master_lock);
-			if (IS_FOREIGN_CALL(c))
+			if (IS_FOREIGN_CALL(c)) {
+				c->redis_hosted_db = rtpe_redis_write->db; // don't delete from foreign DB
 				call_destroy(c);
+			}
 			else {
-				rlog(LOG_WARN, "Redis-Notifier: Ignoring SET received for OWN call: %s\n", rr->element[2]->str);
+				rlog(LOG_WARN, "Redis-Notifier: Ignoring SET received for OWN call: " STR_FORMAT "\n", STR_FMT(&callid));
 				goto err;
 			}
 		}
+
+		redis_select_db(r, r->db);
 	        mutex_unlock(&r->lock);
 
 		// unlock before restoring calls to avoid deadlock in case err happens
@@ -390,12 +394,12 @@ void on_redis_notification(redisAsyncContext *actx, void *reply, void *privdata)
 	if (strncmp(rr->element[3]->str,"del",3)==0) {
 		c = call_get(&callid);
 		if (!c) {
-			rlog(LOG_NOTICE, "Redis-Notifier: DEL did not find call with callid: %s\n", rr->element[2]->str);
+			rlog(LOG_NOTICE, "Redis-Notifier: DEL did not find call with callid: " STR_FORMAT "\n", STR_FMT(&callid));
 			goto err;
 		}
 		rwlock_unlock_w(&c->master_lock);
 		if (!IS_FOREIGN_CALL(c)) {
-			rlog(LOG_WARN, "Redis-Notifier: Ignoring DEL received for an OWN call: %s\n", rr->element[2]->str);
+			rlog(LOG_WARN, "Redis-Notifier: Ignoring DEL received for an OWN call: " STR_FORMAT "\n", STR_FMT(&callid));
 			goto err;
 		}
 		call_destroy(c);
@@ -1751,7 +1755,10 @@ static void json_restore_call(struct redis *r, const str *callid, int foreign) {
 	JsonReader *root_reader =0;
 	JsonParser *parser =0;
 
+	mutex_lock(&r->lock);
 	rr_jsonStr = redis_get(r, REDIS_REPLY_STRING, "GET " PB, STR(callid));
+	mutex_unlock(&r->lock);
+
 	err = "could not retrieve JSON data from redis";
 	if (!rr_jsonStr)
 		goto err1;
@@ -1889,11 +1896,17 @@ err1:
 		g_object_unref (parser);
 	if (rr_jsonStr)
 		freeReplyObject(rr_jsonStr);	
-	log_info_clear();
 	if (err) {
-		rlog(LOG_WARNING, "Failed to restore call ID '" STR_FORMAT_M "' from Redis: %s",
-				STR_FMT_M(callid),
-				err);
+		mutex_lock(&r->lock);
+		if (r->ctx && r->ctx->err)
+			rlog(LOG_WARNING, "Failed to restore call ID '" STR_FORMAT_M "' from Redis: %s (%s)",
+					STR_FMT_M(callid),
+					err, r->ctx->errstr);
+		else
+			rlog(LOG_WARNING, "Failed to restore call ID '" STR_FORMAT_M "' from Redis: %s",
+					STR_FMT_M(callid),
+					err);
+		mutex_unlock(&r->lock);
 		if (c) 
 			call_destroy(c);
 
@@ -1909,6 +1922,7 @@ err1:
 	}
 	if (c)
 		obj_put(c);
+	log_info_clear();
 }
 
 struct thread_ctx {
