@@ -16,6 +16,7 @@
 #include "rtcplib.h"
 #include "ssrc.h"
 #include "sdp.h"
+#include "log_funcs.h"
 
 
 
@@ -67,22 +68,22 @@
 
 
 struct report_block {
-	u_int32_t ssrc;
+	uint32_t ssrc;
 	unsigned char fraction_lost;
 	unsigned char number_lost[3];
-	u_int32_t high_seq_received;
-	u_int32_t jitter;
-	u_int32_t lsr;
-	u_int32_t dlsr;
+	uint32_t high_seq_received;
+	uint32_t jitter;
+	uint32_t lsr;
+	uint32_t dlsr;
 } __attribute__ ((packed));
 
 struct sender_report_packet {
 	struct rtcp_packet rtcp;
-	u_int32_t ntp_msw;
-	u_int32_t ntp_lsw;
-	u_int32_t timestamp;
-	u_int32_t packet_count;
-	u_int32_t octet_count;
+	uint32_t ntp_msw;
+	uint32_t ntp_lsw;
+	uint32_t timestamp;
+	uint32_t packet_count;
+	uint32_t octet_count;
 	struct report_block reports[0];
 } __attribute__ ((packed));
 
@@ -98,7 +99,7 @@ struct sdes_item {
 } __attribute__ ((packed));
 
 struct sdes_chunk {
-	u_int32_t ssrc;
+	uint32_t ssrc;
 	struct sdes_item items[0];
 } __attribute__ ((packed));
 
@@ -109,7 +110,7 @@ struct source_description_packet {
 
 struct bye_packet {
 	struct rtcp_header header;
-	u_int32_t ssrcs[0];
+	uint32_t ssrcs[0];
 } __attribute__ ((packed));
 
 struct app_packet {
@@ -120,14 +121,14 @@ struct app_packet {
 
 struct fb_packet {
 	struct rtcp_packet rtcp;
-	u_int32_t media_ssrc;
+	uint32_t media_ssrc;
 	unsigned char information[0];
 } __attribute__ ((packed));
 
 struct xr_report_block {
-    u_int8_t		 bt;		/**< Block type.		*/
-    u_int8_t		 specific;	/**< Block specific data.	*/
-    u_int16_t		 length;	/**< Block length.		*/
+    uint8_t		 bt;		/**< Block type.		*/
+    uint8_t		 specific;	/**< Block specific data.	*/
+    uint16_t		 length;	/**< Block length.		*/
 } __attribute__ ((packed));
 
 struct xr_packet {
@@ -137,14 +138,14 @@ struct xr_packet {
 
 struct xr_rb_rr_time {
     struct xr_report_block header;
-    u_int32_t		 ntp_msw;	/**< NTP time, seconds part.	*/
-    u_int32_t		 ntp_lsw;	/**< NTP time, fractions part.	*/
+    uint32_t		 ntp_msw;	/**< NTP time, seconds part.	*/
+    uint32_t		 ntp_lsw;	/**< NTP time, fractions part.	*/
 } __attribute__ ((packed));
 
 struct xr_rb_dlrr_item {
-    u_int32_t		 ssrc;		/**< receiver SSRC		*/
-    u_int32_t		 lrr;		/**< last receiver report	*/
-    u_int32_t		 dlrr;		/**< delay since last receiver
+    uint32_t		 ssrc;		/**< receiver SSRC		*/
+    uint32_t		 lrr;		/**< last receiver report	*/
+    uint32_t		 dlrr;		/**< delay since last receiver
 					     report			*/
 } __attribute__ ((packed));
 
@@ -156,54 +157,54 @@ struct xr_rb_dlrr {
 
 struct xr_rb_stats {
     struct xr_report_block header;
-    u_int32_t		 ssrc;		/**< Receiver SSRC		     */
-    u_int16_t		 begin_seq;	/**< Begin RTP sequence reported     */
-    u_int16_t		 end_seq;	/**< End RTP sequence reported       */
-    u_int32_t		 lost;		/**< Number of packet lost in this
+    uint32_t		 ssrc;		/**< Receiver SSRC		     */
+    uint16_t		 begin_seq;	/**< Begin RTP sequence reported     */
+    uint16_t		 end_seq;	/**< End RTP sequence reported       */
+    uint32_t		 lost;		/**< Number of packet lost in this
 					     interval  */
-    u_int32_t		 dup;		/**< Number of duplicated packet in
+    uint32_t		 dup;		/**< Number of duplicated packet in
 					     this interval */
-    u_int32_t		 jitter_min;	/**< Minimum jitter in this interval */
-    u_int32_t		 jitter_max;	/**< Maximum jitter in this interval */
-    u_int32_t		 jitter_mean;	/**< Average jitter in this interval */
-    u_int32_t		 jitter_dev;	/**< Jitter deviation in this
+    uint32_t		 jitter_min;	/**< Minimum jitter in this interval */
+    uint32_t		 jitter_max;	/**< Maximum jitter in this interval */
+    uint32_t		 jitter_mean;	/**< Average jitter in this interval */
+    uint32_t		 jitter_dev;	/**< Jitter deviation in this
 					     interval */
-    u_int32_t		 toh_min:8;	/**< Minimum ToH in this interval    */
-    u_int32_t		 toh_max:8;	/**< Maximum ToH in this interval    */
-    u_int32_t		 toh_mean:8;	/**< Average ToH in this interval    */
-    u_int32_t		 toh_dev:8;	/**< ToH deviation in this interval  */
+    uint32_t		 toh_min:8;	/**< Minimum ToH in this interval    */
+    uint32_t		 toh_max:8;	/**< Maximum ToH in this interval    */
+    uint32_t		 toh_mean:8;	/**< Average ToH in this interval    */
+    uint32_t		 toh_dev:8;	/**< ToH deviation in this interval  */
 } __attribute__ ((packed));
 
 struct xr_rb_voip_metrics {
     struct xr_report_block header;
-    u_int32_t		 ssrc;		/**< Receiver SSRC		*/
-    u_int8_t		 loss_rate;	/**< Packet loss rate		*/
-    u_int8_t		 discard_rate;	/**< Packet discarded rate	*/
-    u_int8_t		 burst_den;	/**< Burst density		*/
-    u_int8_t		 gap_den;	/**< Gap density		*/
-    u_int16_t		 burst_dur;	/**< Burst duration		*/
-    u_int16_t		 gap_dur;	/**< Gap duration		*/
-    u_int16_t		 rnd_trip_delay;/**< Round trip delay		*/
-    u_int16_t		 end_sys_delay; /**< End system delay		*/
-    u_int8_t		 signal_lvl;	/**< Signal level		*/
-    u_int8_t		 noise_lvl;	/**< Noise level		*/
-    u_int8_t		 rerl;		/**< Residual Echo Return Loss	*/
-    u_int8_t		 gmin;		/**< The gap threshold		*/
-    u_int8_t		 r_factor;	/**< Voice quality metric carried
+    uint32_t		 ssrc;		/**< Receiver SSRC		*/
+    uint8_t		 loss_rate;	/**< Packet loss rate		*/
+    uint8_t		 discard_rate;	/**< Packet discarded rate	*/
+    uint8_t		 burst_den;	/**< Burst density		*/
+    uint8_t		 gap_den;	/**< Gap density		*/
+    uint16_t		 burst_dur;	/**< Burst duration		*/
+    uint16_t		 gap_dur;	/**< Gap duration		*/
+    uint16_t		 rnd_trip_delay;/**< Round trip delay		*/
+    uint16_t		 end_sys_delay; /**< End system delay		*/
+    uint8_t		 signal_lvl;	/**< Signal level		*/
+    uint8_t		 noise_lvl;	/**< Noise level		*/
+    uint8_t		 rerl;		/**< Residual Echo Return Loss	*/
+    uint8_t		 gmin;		/**< The gap threshold		*/
+    uint8_t		 r_factor;	/**< Voice quality metric carried
 					     over this RTP session	*/
-    u_int8_t		 ext_r_factor;  /**< Voice quality metric carried
+    uint8_t		 ext_r_factor;  /**< Voice quality metric carried
 					     outside of this RTP session*/
-    u_int8_t		 mos_lq;	/**< Mean Opinion Score for
+    uint8_t		 mos_lq;	/**< Mean Opinion Score for
 					     Listening Quality          */
-    u_int8_t		 mos_cq;	/**< Mean Opinion Score for
+    uint8_t		 mos_cq;	/**< Mean Opinion Score for
 					     Conversation Quality       */
-    u_int8_t		 rx_config;	/**< Receiver configuration	*/
-    u_int8_t		 reserved2;	/**< Not used			*/
-    u_int16_t		 jb_nom;	/**< Current delay by jitter
+    uint8_t		 rx_config;	/**< Receiver configuration	*/
+    uint8_t		 reserved2;	/**< Not used			*/
+    uint16_t		 jb_nom;	/**< Current delay by jitter
 					     buffer			*/
-    u_int16_t		 jb_max;	/**< Maximum delay by jitter
+    uint16_t		 jb_max;	/**< Maximum delay by jitter
 					     buffer			*/
-    u_int16_t		 jb_abs_max;	/**< Maximum possible delay by
+    uint16_t		 jb_abs_max;	/**< Maximum possible delay by
 					     jitter buffer		*/
 }  __attribute__ ((packed));
 
@@ -237,7 +238,7 @@ struct rtcp_process_ctx {
 		struct ssrc_xr_rr_time xr_rr;
 		struct ssrc_xr_dlrr xr_dlrr;
 	} scratch;
-	u_int32_t scratch_common_ssrc;
+	uint32_t scratch_common_ssrc;
 
 	// RTCP syslog output
 	GString *log;
@@ -246,6 +247,9 @@ struct rtcp_process_ctx {
 	// Homer stats
 	GString *json;
 	int json_init_len;
+
+	// verdict
+	unsigned int discard:1;
 };
 // all available methods
 struct rtcp_handler {
@@ -307,6 +311,9 @@ static void transcode_common_wrap(struct rtcp_process_ctx *, struct rtcp_packet
 static void transcode_rr_wrap(struct rtcp_process_ctx *, struct report_block *);
 static void transcode_sr_wrap(struct rtcp_process_ctx *, struct sender_report_packet *);
 
+// RTCP sinks for local RTCP generation
+static void sink_common(struct rtcp_process_ctx *, struct rtcp_packet *);
+
 // homer functions
 static void homer_init(struct rtcp_process_ctx *);
 static void homer_sr(struct rtcp_process_ctx *, struct sender_report_packet *);
@@ -358,6 +365,9 @@ static struct rtcp_handler transcode_handlers = {
 	.rr = transcode_rr,
 	.sr = transcode_sr,
 };
+static struct rtcp_handler sink_handlers = {
+	.common = sink_common,
+};
 static struct rtcp_handler transcode_handlers_wrap = {
 	.common = transcode_common_wrap,
 	.rr = transcode_rr_wrap,
@@ -473,6 +483,7 @@ static const int min_xr_packet_sizes[] = {
 
 
 struct rtcp_handler *rtcp_transcode_handler = &transcode_handlers;
+struct rtcp_handler *rtcp_sink_handler = &sink_handlers;
 
 
 
@@ -622,7 +633,7 @@ static int rtcp_xr(struct rtcp_chain_element *el, struct rtcp_process_ctx *log_c
 		if (!hf)
 			goto next;
 		if (rb->bt < G_N_ELEMENTS(min_xr_packet_sizes) && len < min_xr_packet_sizes[rb->bt]) {
-			ilog(LOG_WARN, "Short RTCP XR block (type %u, %u < %i)", rb->bt, len,
+			ilogs(rtcp, LOG_WARN, "Short RTCP XR block (type %u, %u < %i)", rb->bt, len,
 					min_xr_packet_sizes[rb->bt]);
 			goto next;
 		}
@@ -646,6 +657,7 @@ void rtcp_list_free(GQueue *q) {
 
 
 
+// returns: 0 = ok, forward, -1 = error, drop, 1 = ok, but discard (no forward)
 int rtcp_parse(GQueue *q, struct media_packet *mp) {
 	struct rtcp_header *hdr;
 	struct rtcp_chain_element *el;
@@ -671,7 +683,7 @@ int rtcp_parse(GQueue *q, struct media_packet *mp) {
 			break;
 
 		if (hdr->version != 2) {
-			ilog(LOG_DEBUG, "Unknown RTCP version %u", hdr->version);
+			ilogs(rtcp, LOG_DEBUG, "Unknown RTCP version %u", hdr->version);
 			goto error;
 		}
 
@@ -679,7 +691,7 @@ int rtcp_parse(GQueue *q, struct media_packet *mp) {
 		if (hdr->pt < G_N_ELEMENTS(min_packet_sizes))
 			min_packet_size = min_packet_sizes[hdr->pt];
 		if (len < min_packet_size) {
-			ilog(LOG_WARN, "Invalid RTCP packet type %u (short: %u < %i)",
+			ilogs(rtcp, LOG_WARN, "Invalid RTCP packet type %u (short: %u < %i)",
 					hdr->pt, len, min_packet_size);
 			goto error;
 		}
@@ -687,19 +699,19 @@ int rtcp_parse(GQueue *q, struct media_packet *mp) {
 		el = rtcp_new_element(hdr, len);
 
 		if (hdr->pt >= G_N_ELEMENTS(handler_funcs)) {
-			ilog(LOG_INFO, "Ignoring unknown RTCP packet type %u", hdr->pt);
+			ilogs(rtcp, LOG_INFO, "Ignoring unknown RTCP packet type %u", hdr->pt);
 			goto next;
 		}
 		func = handler_funcs[hdr->pt];
 		if (!func) {
-			ilog(LOG_INFO, "Ignoring unknown RTCP packet type %u", hdr->pt);
+			ilogs(rtcp, LOG_INFO, "Ignoring unknown RTCP packet type %u", hdr->pt);
 			goto next;
 		}
 
-		ilog(LOG_DEBUG, "Calling handler for RTCP packet type %u", hdr->pt);
+		ilogs(rtcp, LOG_DEBUG, "Calling handler for RTCP packet type %u", hdr->pt);
 		ret = func(el, log_ctx);
 		if (ret) {
-			ilog(LOG_WARN, "Failed to handle or parse RTCP packet type %u", hdr->pt);
+			ilogs(rtcp, LOG_WARN, "Failed to handle or parse RTCP packet type %u", hdr->pt);
 			rtcp_ce_free(el);
 			goto error;
 		}
@@ -714,7 +726,7 @@ next:
 	CAH(finish, c, &mp->fsin, &mp->sfd->socket.local, &mp->tv);
 	CAH(destroy);
 
-	return 0;
+	return log_ctx->discard ? 1 : 0;
 
 error:
 	CAH(finish, c, &mp->fsin, &mp->sfd->socket.local, &mp->tv);
@@ -783,7 +795,7 @@ INLINE int check_session_keys(struct crypto_context *c) {
 	return 0;
 
 error:
-	ilog(LOG_ERROR | LOG_FLAG_LIMIT, "%s", err);
+	ilogs(rtcp, LOG_ERROR | LOG_FLAG_LIMIT, "%s", err);
 	return -1;
 }
 
@@ -828,14 +840,14 @@ done:
 	*out = rtcp;
 	return 0;
 error:
-	ilog(LOG_DEBUG | LOG_FLAG_LIMIT, "Error parsing RTCP header: %s", err);
+	ilogs(rtcp, LOG_DEBUG | LOG_FLAG_LIMIT, "Error parsing RTCP header: %s", err);
 	return -1;
 }
 
 /* rfc 3711 section 3.4 */
 int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 	struct rtcp_packet *rtcp;
-	u_int32_t *idx;
+	uint32_t *idx;
 	str to_auth, payload;
 
 	if (G_UNLIKELY(!ssrc_ctx))
@@ -850,9 +862,11 @@ int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 			rtcp->ssrc, ssrc_ctx->srtcp_index);
 	crypto_debug_dump(&payload);
 
+	int prev_len = payload.len;
 	if (!c->params.session_params.unencrypted_srtcp && crypto_encrypt_rtcp(c, rtcp, &payload,
 				ssrc_ctx->srtcp_index))
 		return -1;
+	s->len += payload.len - prev_len;
 
 	crypto_debug_printf(", enc pl: ");
 	crypto_debug_dump(&payload);
@@ -866,10 +880,12 @@ int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 
 	rtp_append_mki(s, c);
 
-	c->params.crypto_suite->hash_rtcp(c, s->s + s->len, &to_auth);
-	crypto_debug_printf(", auth: ");
-	crypto_debug_dump_raw(s->s + s->len, c->params.crypto_suite->srtcp_auth_tag);
-	s->len += c->params.crypto_suite->srtcp_auth_tag;
+	if (c->params.crypto_suite->srtcp_auth_tag) {
+		c->params.crypto_suite->hash_rtcp(c, s->s + s->len, &to_auth);
+		crypto_debug_printf(", auth: ");
+		crypto_debug_dump_raw(s->s + s->len, c->params.crypto_suite->srtcp_auth_tag);
+		s->len += c->params.crypto_suite->srtcp_auth_tag;
+	}
 
 	crypto_debug_finish();
 
@@ -881,7 +897,7 @@ int rtcp_avp2savp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 	struct rtcp_packet *rtcp;
 	str payload, to_auth, to_decrypt, auth_tag;
-	u_int32_t idx, *idx_p;
+	uint32_t idx, *idx_p;
 	char hmac[20];
 	const char *err;
 
@@ -912,6 +928,10 @@ int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 
 	crypto_debug_printf(", idx %" PRIu32, idx);
 
+	if (!auth_tag.len)
+		goto decrypt;
+
+	// authenticate
 	assert(sizeof(hmac) >= auth_tag.len);
 	c->params.crypto_suite->hash_rtcp(c, hmac, &to_auth);
 
@@ -924,6 +944,8 @@ int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 	if (str_memcmp(&auth_tag, hmac))
 		goto error;
 
+decrypt:;
+	int prev_len = to_decrypt.len;
 	if ((idx & 0x80000000ULL)) {
 		if (crypto_decrypt_rtcp(c, rtcp, &to_decrypt, idx & 0x7fffffffULL))
 			return -1;
@@ -934,13 +956,14 @@ int rtcp_savp2avp(str *s, struct crypto_context *c, struct ssrc_ctx *ssrc_ctx) {
 
 	*s = to_auth;
 	s->len -= sizeof(idx);
+	s->len -= prev_len - to_decrypt.len;
 
 	crypto_debug_finish();
 
 	return 0;
 
 error:
-	ilog(LOG_WARNING | LOG_FLAG_LIMIT, "Discarded invalid SRTCP packet: %s", err);
+	ilogs(rtcp, LOG_WARNING | LOG_FLAG_LIMIT, "Discarded invalid SRTCP packet: %s", err);
 	return -1;
 }
 
@@ -1094,7 +1117,10 @@ static void homer_sdes_item(struct rtcp_process_ctx *ctx, const struct sdes_chun
 				g_string_append(ctx->json, "\\t");
 				break;
 			default:
-				g_string_append_c(ctx->json, data[i]);
+				if (data[i] < ' ' || data[i] > 126)
+					g_string_append_c(ctx->json, '_');
+				else
+					g_string_append_c(ctx->json, data[i]);
 				break;
 		}
 	}
@@ -1273,7 +1299,7 @@ static void transcode_common(struct rtcp_process_ctx *ctx, struct rtcp_packet *c
 		return;
 	// forward SSRC mapping
 	common->ssrc = htonl(ctx->mp->ssrc_in->ssrc_map_out);
-	ilog(LOG_DEBUG, "Substituting RTCP header SSRC from %s%x%s to %x",
+	ilogs(rtcp, LOG_DEBUG, "Substituting RTCP header SSRC from %s%x%s to %x",
 		FMT_M(ctx->scratch_common_ssrc), ctx->mp->ssrc_in->ssrc_map_out);
 }
 static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr) {
@@ -1281,14 +1307,20 @@ static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr)
 		return;
 	if (ctx->scratch.rr.from != ctx->mp->ssrc_in->parent->h.ssrc)
 		return;
+	if (!ctx->mp->media)
+		return;
 
 	// reverse SSRC mapping
-	struct ssrc_ctx *map_ctx = get_ssrc_ctx(ctx->scratch.rr.ssrc, ctx->mp->call->ssrc_hash,
+	struct ssrc_ctx *map_ctx = get_ssrc_ctx(ctx->scratch.rr.ssrc, ctx->mp->media->monologue->ssrc_hash,
 			SSRC_DIR_OUTPUT, ctx->mp->media->monologue);
 	rr->ssrc = htonl(map_ctx->ssrc_map_out);
 
+	if (!ctx->mp->media_out)
+		return;
+
 	// for reception stats
-	struct ssrc_ctx *input_ctx = get_ssrc_ctx(map_ctx->ssrc_map_out, ctx->mp->call->ssrc_hash,
+	struct ssrc_ctx *input_ctx = get_ssrc_ctx(map_ctx->ssrc_map_out,
+			ctx->mp->media_out->monologue->ssrc_hash,
 			SSRC_DIR_INPUT, NULL);
 
 	// substitute our own values
@@ -1304,7 +1336,7 @@ static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr)
 	unsigned int dupes = atomic64_get(&input_ctx->duplicates);
 	unsigned int tot_lost = lost - dupes; // can be negative/rollover
 
-	ilog(LOG_DEBUG, "Substituting RTCP RR SSRC from %s%x%s to %x: %u packets, %u lost, %u duplicates",
+	ilogs(rtcp, LOG_DEBUG, "Substituting RTCP RR SSRC from %s%x%s to %x: %u packets, %u lost, %u duplicates",
 		FMT_M(ctx->scratch.rr.ssrc), map_ctx->ssrc_map_out,
 		packets, lost, dupes);
 
@@ -1329,8 +1361,7 @@ static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr)
 out:
 	if (input_ctx)
 		obj_put(&input_ctx->parent->h);
-	if (map_ctx)
-		obj_put(&map_ctx->parent->h);
+	obj_put(&map_ctx->parent->h);
 }
 static void transcode_sr(struct rtcp_process_ctx *ctx, struct sender_report_packet *sr) {
 	if (!ctx->mp->ssrc_in)
@@ -1358,17 +1389,20 @@ static void transcode_sr(struct rtcp_process_ctx *ctx, struct sender_report_pack
 static void transcode_common_wrap(struct rtcp_process_ctx *ctx, struct rtcp_packet *common) {
 	if (!ctx->mp->media->rtcp_handler)
 		return;
-	ctx->mp->media->rtcp_handler->common(ctx, common);
+	if (ctx->mp->media->rtcp_handler->common)
+		ctx->mp->media->rtcp_handler->common(ctx, common);
 }
 static void transcode_rr_wrap(struct rtcp_process_ctx *ctx, struct report_block *rr) {
 	if (!ctx->mp->media->rtcp_handler)
 		return;
-	ctx->mp->media->rtcp_handler->rr(ctx, rr);
+	if (ctx->mp->media->rtcp_handler->rr)
+		ctx->mp->media->rtcp_handler->rr(ctx, rr);
 }
 static void transcode_sr_wrap(struct rtcp_process_ctx *ctx, struct sender_report_packet *sr) {
 	if (!ctx->mp->media->rtcp_handler)
 		return;
-	ctx->mp->media->rtcp_handler->sr(ctx, sr);
+	if (ctx->mp->media->rtcp_handler->sr)
+		ctx->mp->media->rtcp_handler->sr(ctx, sr);
 }
 
 
@@ -1381,7 +1415,10 @@ void rtcp_init() {
 
 
 
-GString *rtcp_sender_report(uint32_t ssrc, uint32_t ts, uint32_t packets, uint32_t octets, GQueue *rrs) {
+static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
+		uint32_t ssrc, uint32_t ssrc_out, uint32_t ts, uint32_t packets, uint32_t octets, GQueue *rrs,
+		GQueue *srrs)
+{
 	GString *ret = g_string_sized_new(128);
 	g_string_set_size(ret, sizeof(struct sender_report_packet));
 	struct sender_report_packet *sr = (void *) ret->str;
@@ -1396,11 +1433,21 @@ GString *rtcp_sender_report(uint32_t ssrc, uint32_t ts, uint32_t packets, uint32
 		.packet_count = htonl(packets),
 		.octet_count = htonl(octets),
 	};
+	if (ssr) {
+		*ssr = (struct ssrc_sender_report) {
+			.ssrc = ssrc_out,
+			.ntp_msw = rtpe_now.tv_sec + 2208988800,
+			.ntp_lsw = (4294967295ULL * rtpe_now.tv_usec) / 1000000ULL,
+			.timestamp = ts, // XXX calculate from rtpe_now instead
+			.packet_count = packets,
+			.octet_count = octets,
+		};
+	}
 
 	// receiver reports
 	int i = 0, n = 0;
-	for (GList *l = rrs->head; l; l = l->next) {
-		struct ssrc_ctx *s = l->data;
+	while (rrs->length) {
+		struct ssrc_ctx *s = g_queue_pop_head(rrs);
 		if (i < 30) {
 			struct report_block *rr = (void *) ret->str + ret->len;
 			g_string_set_size(ret, ret->len + sizeof(*rr));
@@ -1417,6 +1464,7 @@ GString *rtcp_sender_report(uint32_t ssrc, uint32_t ts, uint32_t packets, uint32
 				tv_diff = timeval_diff(&rtpe_now, &si->received);
 				ntp_middle_bits = si->ntp_middle_bits;
 			}
+			uint32_t jitter = se->jitter;
 			mutex_unlock(&se->h.lock);
 
 			uint64_t lost = atomic64_get(&s->packets_lost);
@@ -1431,8 +1479,24 @@ GString *rtcp_sender_report(uint32_t ssrc, uint32_t ts, uint32_t packets, uint32
 				.high_seq_received = htonl(atomic64_get(&s->last_seq)),
 				.lsr = htonl(ntp_middle_bits),
 				.dlsr = htonl(tv_diff * 65536 / 1000000),
+				.jitter = htonl(jitter >> 4),
 			};
-			// XXX jitter
+
+			if (srrs) {
+				struct ssrc_receiver_report *srr = g_slice_alloc(sizeof(*srr));
+				*srr = (struct ssrc_receiver_report) {
+					.from = ssrc_out,
+					.ssrc = s->ssrc_map_out ? : s->parent->h.ssrc,
+					.fraction_lost = lost * 256 / (tot + lost),
+					.packets_lost = lost,
+					.high_seq_received = atomic64_get(&s->last_seq),
+					.lsr = ntp_middle_bits,
+					.dlsr = tv_diff * 65536 / 1000000,
+					.jitter = jitter >> 4,
+				};
+				g_queue_push_tail(srrs, srr);
+			}
+
 			n++;
 		}
 		ssrc_ctx_put(&s);
@@ -1479,14 +1543,73 @@ void rtcp_receiver_reports(GQueue *out, struct ssrc_hash *hash, struct call_mono
 	rwlock_lock_r(&hash->lock);
 	for (GList *l = hash->q.head; l; l = l->next) {
 		struct ssrc_entry_call *e = l->data;
-		ilog(LOG_DEBUG, "xxxxx %x %i %i %p %p %p", e->h.ssrc, (int) atomic64_get(&e->input_ctx.packets), (int) atomic64_get(&e->output_ctx.packets), ml, e->input_ctx.ref, e->output_ctx.ref);
+		//ilog(LOG_DEBUG, "xxxxx %x %i %i %p %p %p", e->h.ssrc, (int) atomic64_get(&e->input_ctx.packets), (int) atomic64_get(&e->output_ctx.packets), ml, e->input_ctx.ref, e->output_ctx.ref);
 		struct ssrc_ctx *i = &e->input_ctx;
 		if (i->ref != ml)
 			continue;
 		if (!atomic64_get(&i->packets))
 			continue;
 
-		g_queue_push_tail(out, ssrc_ctx_get(i));
+		ssrc_ctx_hold(i);
+		g_queue_push_tail(out, i);
 	}
 	rwlock_unlock_r(&hash->lock);
 }
+
+
+// call must be locked in R
+void rtcp_send_report(struct call_media *media, struct ssrc_ctx *ssrc_out) {
+	// figure out where to send it
+	struct packet_stream *ps = media->streams.head->data;
+	if (MEDIA_ISSET(media, RTCP_MUX))
+		;
+	else if (!media->streams.head->next)
+		;
+	else {
+		struct packet_stream *next_ps = media->streams.head->next->data;
+		if (PS_ISSET(next_ps, RTCP))
+			ps = next_ps;
+	}
+
+	struct call_media *other_media = NULL;
+	if (ps->rtp_sink)
+		other_media = ps->rtp_sink->media;
+	else if (ps->rtcp_sink)
+		other_media = ps->rtcp_sink->media;
+
+	log_info_stream_fd(ps->selected_sfd);
+
+	GQueue rrs = G_QUEUE_INIT;
+	rtcp_receiver_reports(&rrs, media->monologue->ssrc_hash, ps->media->monologue);
+
+	ilogs(rtcp, LOG_DEBUG, "Generating and sending RTCP SR for %x and up to %i source(s)",
+			ssrc_out->parent->h.ssrc, rrs.length);
+
+	struct ssrc_sender_report ssr;
+	GQueue srrs = G_QUEUE_INIT;
+
+	GString *sr = rtcp_sender_report(&ssr, ssrc_out->parent->h.ssrc,
+			ssrc_out->ssrc_map_out ? : ssrc_out->parent->h.ssrc,
+			atomic64_get(&ssrc_out->last_ts),
+			atomic64_get(&ssrc_out->packets),
+			atomic64_get(&ssrc_out->octets),
+			&rrs, &srrs);
+
+	socket_sendto(&ps->selected_sfd->socket, sr->str, sr->len, &ps->endpoint);
+	g_string_free(sr, TRUE);
+
+	if (other_media)
+		ssrc_sender_report(other_media, &ssr, &rtpe_now);
+	struct ssrc_receiver_report *srr;
+	while ((srr = g_queue_pop_head(&srrs))) {
+		if (other_media)
+			ssrc_receiver_report(other_media, srr, &rtpe_now);
+		g_slice_free1(sizeof(*srr), srr);
+	}
+}
+
+
+
+static void sink_common(struct rtcp_process_ctx *ctx, struct rtcp_packet *common) {
+	ctx->discard = 1;
+}
