@@ -29,15 +29,17 @@ struct network_address {
 struct sdp_origin {
 	str username;
 	str session_id;
-	str version;
+	str version_str;
 	struct network_address address;
-	int parsed:1;
+	unsigned long long version_num;
+	size_t version_output_pos;
+	unsigned int parsed:1;
 };
 
 struct sdp_connection {
 	str s;
 	struct network_address address;
-	int parsed:1;
+	unsigned int parsed:1;
 };
 
 struct sdp_attributes {
@@ -51,6 +53,7 @@ struct sdp_attributes {
 struct sdp_session {
 	str s;
 	struct sdp_origin origin;
+	str session_name;
 	struct sdp_connection connection;
 	int rr, rs;
 	struct sdp_attributes attributes;
@@ -95,7 +98,7 @@ struct attribute_candidate {
 	str related_port_str;
 
 	struct ice_candidate cand_parsed;
-	int parsed:1;
+	unsigned int parsed:1;
 };
 
 struct attribute_crypto {
@@ -113,19 +116,19 @@ struct attribute_crypto {
 	str master_key;
 	str salt;
 	char key_salt_buf[SRTP_MAX_MASTER_KEY_LEN + SRTP_MAX_MASTER_SALT_LEN];
-	u_int64_t lifetime;
+	uint64_t lifetime;
 	unsigned char mki[256];
 	unsigned int mki_len;
-	int unencrypted_srtcp:1,
-	    unencrypted_srtp:1,
-	    unauthenticated_srtp:1;
+	unsigned int unencrypted_srtcp:1,
+	             unencrypted_srtp:1,
+	             unauthenticated_srtp:1;
 };
 
 struct attribute_ssrc {
 	str id_str;
 	str attr_str;
 
-	u_int32_t id;
+	uint32_t id;
 	str attr;
 	str value;
 };
@@ -164,6 +167,13 @@ struct attribute_rtpmap {
 	struct rtp_payload_type rtp_pt;
 };
 
+struct attribute_rtcp_fb {
+	str payload_type_str;
+	str value;
+
+	unsigned int payload_type;
+};
+
 struct attribute_fmtp {
 	str payload_type_str;
 	str format_parms_str;
@@ -253,6 +263,7 @@ struct sdp_attribute {	/* example: a=rtpmap:8 PCMA/8000 */
 		struct attribute_fingerprint fingerprint;
 		struct attribute_setup setup;
 		struct attribute_rtpmap rtpmap;
+		struct attribute_rtcp_fb rtcp_fb;
 		struct attribute_fmtp fmtp;
 		struct attribute_t38faxudpec t38faxudpec;
 		int i;
@@ -339,9 +350,10 @@ static int parse_origin(str *value_str, struct sdp_origin *output) {
 
 	EXTRACT_TOKEN(username);
 	EXTRACT_TOKEN(session_id);
-	EXTRACT_TOKEN(version);
+	EXTRACT_TOKEN(version_str);
 	EXTRACT_NETWORK_ADDRESS_NF(address);
 
+	output->version_num = strtoull(output->version_str.s, NULL, 10);
 	output->parsed = 1;
 	return 0;
 }
@@ -449,7 +461,7 @@ static int parse_attribute_crypto(struct sdp_attribute *output) {
 	unsigned int b64_save = 0;
 	gsize ret;
 	str s;
-	u_int32_t u32;
+	uint32_t u32;
 	const char *err;
 
 	output->attr = ATTR_CRYPTO;
@@ -744,6 +756,28 @@ static int parse_attribute_setup(struct sdp_attribute *output) {
 	return 0;
 }
 
+static int parse_attribute_rtcp_fb(struct sdp_attribute *output) {
+	PARSE_DECL;
+	struct attribute_rtcp_fb *a;
+
+	output->attr = ATTR_RTCP_FB;
+	a = &output->u.rtcp_fb;
+
+	PARSE_INIT;
+	EXTRACT_TOKEN(u.rtcp_fb.payload_type_str);
+	a->value = *value_str;
+
+	if (!str_cmp(&output->u.rtcp_fb.value, "*"))
+		a->payload_type = -1;
+	else {
+		a->payload_type = str_to_i(&a->payload_type_str, -1);
+		if (a->payload_type == -1)
+			return -1;
+	}
+
+	return 0;
+}
+
 static int parse_attribute_rtpmap(struct sdp_attribute *output) {
 	PARSE_DECL;
 	char *ep;
@@ -803,7 +837,7 @@ static int parse_attribute_fmtp(struct sdp_attribute *output) {
 
 	PARSE_INIT;
 	EXTRACT_TOKEN(u.fmtp.payload_type_str);
-	output->u.fmtp.format_parms_str = *value_str;
+	a->format_parms_str = *value_str;
 
 	a->payload_type = str_to_i(&a->payload_type_str, -1);
 	if (a->payload_type == -1)
@@ -979,7 +1013,7 @@ static int parse_attribute(struct sdp_attribute *a) {
 			a->attr = ATTR_END_OF_CANDIDATES;
 			break;
 		case CSH_LOOKUP("rtcp-fb"):
-			a->attr = ATTR_RTCP_FB;
+			ret = parse_attribute_rtcp_fb(a);
 			break;
 		case CSH_LOOKUP("T38FaxVersion"):
 			ret = parse_attribute_int(a, ATTR_T38FAXVERSION, -1);
@@ -1144,7 +1178,8 @@ new_session:
 
 				/* attr_queue = g_hash_table_lookup_queue_new(attrs->name_lists_hash, &attr->name);
 				g_queue_push_tail(attr_queue, attr); */
-				attr_queue = g_hash_table_lookup_queue_new(attrs->id_lists_hash, &attr->attr);
+				attr_queue = g_hash_table_lookup_queue_new(attrs->id_lists_hash, &attr->attr,
+						NULL);
 				g_queue_push_tail(attr_queue, attr);
 
 				break;
@@ -1170,6 +1205,12 @@ new_session:
 				break;
 
 			case 's':
+				errstr = "s= line found within media section";
+				if (media)
+					goto error;
+				session->session_name = value_str;
+				break;
+
 			case 'i':
 			case 'u':
 			case 'e':
@@ -1259,7 +1300,7 @@ static int fill_endpoint(struct endpoint *ep, const struct sdp_media *media, str
 
 static int __rtp_payload_types(struct stream_params *sp, struct sdp_media *media)
 {
-	GHashTable *ht_rtpmap, *ht_fmtp;
+	GHashTable *ht_rtpmap, *ht_fmtp, *ht_rtcp_fb;
 	GQueue *q;
 	GList *ql;
 	struct sdp_attribute *attr;
@@ -1284,6 +1325,16 @@ static int __rtp_payload_types(struct stream_params *sp, struct sdp_media *media
 		attr = ql->data;
 		g_hash_table_insert(ht_fmtp, &attr->u.fmtp.payload_type, &attr->u.fmtp.format_parms_str);
 	}
+	// do the same for a=rtcp-fb
+	ht_rtcp_fb = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) g_queue_free);
+	q = attr_list_get_by_id(&media->attributes, ATTR_RTCP_FB);
+	for (ql = q ? q->head : NULL; ql; ql = ql->next) {
+		attr = ql->data;
+		if (attr->u.rtcp_fb.payload_type == -1)
+			continue;
+		GQueue *rq = g_hash_table_lookup_queue_new(ht_rtcp_fb, GINT_TO_POINTER(attr->u.rtcp_fb.payload_type), NULL);
+		g_queue_push_tail(rq, &attr->u.rtcp_fb.value);
+	}
 
 	/* then go through the format list and associate */
 	for (ql = media->format_list.head; ql; ql = ql->next) {
@@ -1314,6 +1365,13 @@ static int __rtp_payload_types(struct stream_params *sp, struct sdp_media *media
 		s = g_hash_table_lookup(ht_fmtp, &i);
 		if (s)
 			pt->format_parameters = *s;
+		GQueue *rq = g_hash_table_lookup(ht_rtcp_fb, GINT_TO_POINTER(i));
+		if (rq) {
+			// steal the list contents and free the list
+			pt->rtcp_fb = *rq;
+			g_queue_init(rq);
+			g_hash_table_remove(ht_rtcp_fb, GINT_TO_POINTER(i)); // frees `rq`
+		}
 
 		// fill in ptime
 		if (sp->ptime)
@@ -1332,6 +1390,7 @@ error:
 out:
 	g_hash_table_destroy(ht_rtpmap);
 	g_hash_table_destroy(ht_fmtp);
+	g_hash_table_destroy(ht_rtcp_fb);
 	return ret;
 }
 
@@ -1372,7 +1431,7 @@ no_cand:
 		SP_SET(sp, TRICKLE_ICE);
 
 	if (attr_get_by_id(&media->attributes, ATTR_ICE_LITE))
-		SP_SET(sp, ICE_LITE);
+		SP_SET(sp, ICE_LITE_PEER);
 
 	attr = attr_get_by_id_m_s(media, ATTR_ICE_PWD);
 	if (attr)
@@ -1457,8 +1516,15 @@ int sdp_streams(const GQueue *sessions, GQueue *streams, struct sdp_ng_flags *fl
 			sp = g_slice_alloc0(sizeof(*sp));
 			sp->index = ++num;
 
+			__sdp_ice(sp, media);
+			if (SP_ISSET(sp, ICE)) {
+				// ignore "received from" (SIP-source-address) when ICE is in use
+				flags->trust_address = 1;
+			}
+
 			errstr = "No address info found for stream";
-			if (fill_endpoint(&sp->rtp_endpoint, media, flags, NULL, media->port_num))
+			if (!flags->fragment
+					&& fill_endpoint(&sp->rtp_endpoint, media, flags, NULL, media->port_num))
 				goto error;
 
 			sp->consecutive_ports = media->port_count;
@@ -1536,6 +1602,7 @@ int sdp_streams(const GQueue *sessions, GQueue *streams, struct sdp_ng_flags *fl
 				sp->fingerprint.hash_func = attr->u.fingerprint.hash_func;
 				memcpy(sp->fingerprint.digest, attr->u.fingerprint.fingerprint,
 						sp->fingerprint.hash_func->num_bytes);
+				sp->fingerprint.digest_len = sp->fingerprint.hash_func->num_bytes;
 			}
 
 			// OSRTP (RFC 8643)
@@ -1555,7 +1622,6 @@ int sdp_streams(const GQueue *sessions, GQueue *streams, struct sdp_ng_flags *fl
 			if (attr_get_by_id(&media->attributes, ATTR_RTCP_FB))
 				SP_SET(sp, RTCP_FB);
 
-			__sdp_ice(sp, media);
 			__sdp_t38(sp, media);
 
 			/* determine RTCP endpoint */
@@ -1610,6 +1676,29 @@ INLINE void chopper_append_c(struct sdp_chopper *c, const char *s) {
 INLINE void chopper_append_str(struct sdp_chopper *c, const str *s) {
 	chopper_append(c, s->s, s->len);
 }
+static void chopper_replace(struct sdp_chopper *c, str *old, size_t *old_pos,
+		const char *repl, size_t repl_len)
+{
+	// adjust for offsets created within this run
+	*old_pos += c->offset;
+	// is our new value longer?
+	if (repl_len > old->len) {
+		// overwrite + insert
+		g_string_overwrite_len(c->output, *old_pos, repl, old->len);
+		g_string_insert(c->output, *old_pos + old->len, repl + old->len);
+		c->offset += repl_len - old->len;
+		old->len = repl_len;
+	}
+	else {
+		// overwrite + optional erase
+		g_string_overwrite(c->output, *old_pos, repl);
+		if (repl_len < old->len) {
+			g_string_erase(c->output, *old_pos + repl_len, old->len - repl_len);
+			c->offset -= old->len - repl_len;
+			old->len = repl_len;
+		}
+	}
+}
 
 #define chopper_append_printf(c, f...) g_string_append_printf((c)->output, f)
 
@@ -1719,14 +1808,17 @@ static void insert_codec_parameters(struct sdp_chopper *chop, struct call_media
 		chopper_append_printf(chop, "a=rtpmap:%u " STR_FORMAT "\r\n",
 				pt->payload_type,
 				STR_FMT(&pt->encoding_with_params));
-	}
-	for (GList *l = cm->codecs_prefs_recv.head; l; l = l->next) {
-		struct rtp_payload_type *pt = l->data;
-		if (!pt->format_parameters.len)
-			continue;
-		chopper_append_printf(chop, "a=fmtp:%u " STR_FORMAT "\r\n",
-				pt->payload_type,
-				STR_FMT(&pt->format_parameters));
+		if (pt->format_parameters.len) {
+			chopper_append_printf(chop, "a=fmtp:%u " STR_FORMAT "\r\n",
+					pt->payload_type,
+					STR_FMT(&pt->format_parameters));
+		}
+		for (GList *l = pt->rtcp_fb.head; l; l = l->next) {
+			str *fb = l->data;
+			chopper_append_printf(chop, "a=rtcp-fb:%u " STR_FORMAT "\r\n",
+					pt->payload_type,
+					STR_FMT(fb));
+		}
 	}
 }
 
@@ -1800,23 +1892,31 @@ warn:
 	return 0;
 }
 
-static int insert_ice_address(struct sdp_chopper *chop, struct stream_fd *sfd) {
+static int insert_ice_address(struct sdp_chopper *chop, struct stream_fd *sfd, struct sdp_ng_flags *flags) {
 	char buf[64];
 	int len;
 
-	call_stream_address46(buf, sfd->stream, SAF_ICE, &len, sfd->local_intf, 0);
+	if (!is_addr_unspecified(&flags->parsed_media_address))
+		len = sprintf(buf, "%s",
+				sockaddr_print_buf(&flags->parsed_media_address));
+	else
+		call_stream_address46(buf, sfd->stream, SAF_ICE, &len, sfd->local_intf, 0);
 	chopper_append(chop, buf, len);
 	chopper_append_printf(chop, " %u", sfd->socket.local.port);
 
 	return 0;
 }
 
-static int insert_raddr_rport(struct sdp_chopper *chop, struct stream_fd *sfd) {
+static int insert_raddr_rport(struct sdp_chopper *chop, struct stream_fd *sfd, struct sdp_ng_flags *flags) {
         char buf[64];
         int len;
 
 	chopper_append_c(chop, " raddr ");
-	call_stream_address46(buf, sfd->stream, SAF_ICE, &len, sfd->local_intf, 0);
+	if (!is_addr_unspecified(&flags->parsed_media_address))
+		len = sprintf(buf, "%s",
+				sockaddr_print_buf(&flags->parsed_media_address));
+	else
+		call_stream_address46(buf, sfd->stream, SAF_ICE, &len, sfd->local_intf, 0);
 	chopper_append(chop, buf, len);
 	chopper_append_c(chop, " rport ");
 	chopper_append_printf(chop, "%u", sfd->socket.local.port);
@@ -1830,11 +1930,6 @@ static int replace_network_address(struct sdp_chopper *chop, struct network_addr
 {
 	char buf[64];
 	int len;
-	struct packet_stream *sink = packet_stream_sink(ps);
-
-	if (is_addr_unspecified(&address->parsed)
-			&& !(sink && is_trickle_ice_address(&sink->advertised_endpoint)))
-		return 0;
 
 	if (copy_up_to(chop, &address->address_type))
 		return -1;
@@ -1892,19 +1987,21 @@ static int process_session_attributes(struct sdp_chopper *chop, struct sdp_attri
 			case ATTR_ICE_PWD:
 			case ATTR_ICE_OPTIONS:
 			case ATTR_ICE_LITE:
-				if (!flags->ice_remove && !flags->ice_force)
+				if (flags->ice_option != ICE_REMOVE && flags->ice_option != ICE_FORCE
+						&& flags->ice_option != ICE_DEFAULT)
 					break;
 				goto strip;
 
 			case ATTR_CANDIDATE:
-				if (flags->ice_force_relay) {
+				if (flags->ice_option == ICE_FORCE_RELAY) {
 					if ((attr->u.candidate.type_str.len == 5) &&
 					    (strncasecmp(attr->u.candidate.type_str.s, "relay", 5) == 0))
 						goto strip;
 					else
 						break;
 				}
-				if (!flags->ice_remove && !flags->ice_force)
+				if (flags->ice_option != ICE_REMOVE && flags->ice_option != ICE_FORCE
+						&& flags->ice_option != ICE_DEFAULT)
 					break;
 				goto strip;
 
@@ -1965,12 +2062,13 @@ static int process_media_attributes(struct sdp_chopper *chop, struct sdp_media *
 			case ATTR_ICE_LITE:
 				if (MEDIA_ISSET(media, PASSTHRU))
 					break;
-				if (!flags->ice_remove && !flags->ice_force)
+				if (flags->ice_option != ICE_REMOVE && flags->ice_option != ICE_FORCE
+						&& flags->ice_option != ICE_DEFAULT)
 					break;
 				goto strip;
 
 			case ATTR_CANDIDATE:
-				if (flags->ice_force_relay) {
+				if (flags->ice_option == ICE_FORCE_RELAY) {
 					if ((attr->u.candidate.type_str.len == 5) &&
 					    (strncasecmp(attr->u.candidate.type_str.s, "relay", 5) == 0))
 						goto strip;
@@ -1979,13 +2077,14 @@ static int process_media_attributes(struct sdp_chopper *chop, struct sdp_media *
 				}
 				if (MEDIA_ISSET(media, PASSTHRU))
 					break;
-				if (!flags->ice_remove && !flags->ice_force)
+				if (flags->ice_option != ICE_REMOVE && flags->ice_option != ICE_FORCE
+						&& flags->ice_option != ICE_DEFAULT)
 					break;
 				goto strip;
 
 			case ATTR_RTCP:
 			case ATTR_RTCP_MUX:
-				if (flags->ice_force_relay)
+				if (flags->ice_option == ICE_FORCE_RELAY)
 					break;
 				goto strip;
 
@@ -2004,7 +2103,16 @@ static int process_media_attributes(struct sdp_chopper *chop, struct sdp_media *
 
 			case ATTR_RTPMAP:
 			case ATTR_FMTP:
+				if (media->codecs_prefs_recv.length > 0)
+					goto strip;
+				break;
 			case ATTR_PTIME:
+				if (media->ptime)
+					goto strip;
+				break;
+			case ATTR_RTCP_FB:
+				if (attr->u.rtcp_fb.payload_type == -1)
+					break; // leave this one alone
 				if (media->codecs_prefs_recv.length > 0)
 					goto strip;
 				break;
@@ -2038,7 +2146,7 @@ static void new_priority(struct sdp_media *media, enum ice_candidate_type type,
 {
 	GQueue *cands;
 	unsigned int lpref, tpref;
-	u_int32_t prio;
+	uint32_t prio;
 	GList *l;
 	struct sdp_attribute *a;
 	struct attribute_candidate *c;
@@ -2080,7 +2188,8 @@ out:
 }
 
 static void insert_candidate(struct sdp_chopper *chop, struct stream_fd *sfd,
-		unsigned int type_pref, unsigned int local_pref, enum ice_candidate_type type)
+		unsigned int type_pref, unsigned int local_pref, enum ice_candidate_type type,
+		struct sdp_ng_flags *flags)
 {
 	unsigned long priority;
 	struct packet_stream *ps = sfd->stream;
@@ -2093,24 +2202,25 @@ static void insert_candidate(struct sdp_chopper *chop, struct stream_fd *sfd,
 	chopper_append_c(chop, "a=candidate:");
 	chopper_append_str(chop, &ifa->ice_foundation);
 	chopper_append_printf(chop, " %u UDP %lu ", ps->component, priority);
-	insert_ice_address(chop, sfd);
+	insert_ice_address(chop, sfd, flags);
 	chopper_append_c(chop, " typ ");
 	chopper_append_c(chop, ice_candidate_type_str(type));
 	/* raddr and rport are required for non-host candidates: rfc5245 section-15.1 */
 	if(type != ICT_HOST)
-		insert_raddr_rport(chop, sfd);
+		insert_raddr_rport(chop, sfd, flags);
 	chopper_append_c(chop, "\r\n");
 }
 
 static void insert_sfd_candidates(struct sdp_chopper *chop, struct packet_stream *ps,
-		unsigned int type_pref, unsigned int local_pref, enum ice_candidate_type type)
+		unsigned int type_pref, unsigned int local_pref, enum ice_candidate_type type,
+		struct sdp_ng_flags *flags)
 {
 	GList *l;
 	struct stream_fd *sfd;
 
 	for (l = ps->sfds.head; l; l = l->next) {
 		sfd = l->data;
-		insert_candidate(chop, sfd, type_pref, local_pref, type);
+		insert_candidate(chop, sfd, type_pref, local_pref, type, flags);
 
 		if (local_pref != -1)
 			local_pref++;
@@ -2130,7 +2240,7 @@ static void insert_candidates(struct sdp_chopper *chop, struct packet_stream *rt
 	media = rtp->media;
 
 	cand_type = ICT_HOST;
-	if (flags->ice_force_relay)
+	if (flags->ice_option == ICE_FORCE_RELAY)
 		cand_type = ICT_RELAY;
 	if (MEDIA_ISSET(media, PASSTHRU))
 		new_priority(sdp_media, cand_type, &type_pref, &local_pref);
@@ -2143,9 +2253,9 @@ static void insert_candidates(struct sdp_chopper *chop, struct packet_stream *rt
 
 	if (ag && AGENT_ISSET(ag, COMPLETED)) {
 		ifa = rtp->selected_sfd->local_intf;
-		insert_candidate(chop, rtp->selected_sfd, type_pref, ifa->unique_id, cand_type);
+		insert_candidate(chop, rtp->selected_sfd, type_pref, ifa->unique_id, cand_type, flags);
 		if (rtcp) /* rtcp-mux only possible in answer */
-			insert_candidate(chop, rtcp->selected_sfd, type_pref, ifa->unique_id, cand_type);
+			insert_candidate(chop, rtcp->selected_sfd, type_pref, ifa->unique_id, cand_type, flags);
 
 		if (flags->opmode == OP_OFFER && AGENT_ISSET(ag, CONTROLLING)) {
 			GQueue rc;
@@ -2165,10 +2275,10 @@ static void insert_candidates(struct sdp_chopper *chop, struct packet_stream *rt
 		return;
 	}
 
-	insert_sfd_candidates(chop, rtp, type_pref, local_pref, cand_type);
+	insert_sfd_candidates(chop, rtp, type_pref, local_pref, cand_type, flags);
 
 	if (rtcp) /* rtcp-mux only possible in answer */
-		insert_sfd_candidates(chop, rtcp, type_pref, local_pref, cand_type);
+		insert_sfd_candidates(chop, rtcp, type_pref, local_pref, cand_type, flags);
 }
 
 static void insert_dtls(struct call_media *media, struct sdp_chopper *chop) {
@@ -2183,11 +2293,28 @@ static void insert_dtls(struct call_media *media, struct sdp_chopper *chop) {
 	if (!call->dtls_cert || !MEDIA_ISSET(media, DTLS) || MEDIA_ISSET(media, PASSTHRU))
 		return;
 
-	hf = call->dtls_cert->fingerprint.hash_func;
+	hf = media->fp_hash_func;
+	if (!hf)
+		hf = media->fingerprint.hash_func;
+
+	struct dtls_fingerprint *fp = NULL;
+	for (GList *l = call->dtls_cert->fingerprints.head; l; l = l->next) {
+		fp = l->data;
+		if (!hf)
+			break;
+		if (!strcasecmp(hf->name, fp->hash_func->name))
+			break;
+		fp = NULL;
+	}
+	if (!fp) // use first if no match
+		fp = call->dtls_cert->fingerprints.head->data;
+
+	hf = fp->hash_func;
+	media->fp_hash_func = hf;
 
 	assert(hf->num_bytes > 0);
 
-	p = call->dtls_cert->fingerprint.digest;
+	p = fp->digest;
 	o = hexbuf;
 	for (i = 0; i < hf->num_bytes; i++)
 		o += sprintf(o, "%02X:", *p++);
@@ -2268,13 +2395,53 @@ static void insert_rtcp_attr(struct sdp_chopper *chop, struct packet_stream *ps,
 	if (flags->full_rtcp_attr) {
 		char buf[64];
 		int len;
-		call_stream_address46(buf, ps, SAF_NG, &len, NULL, 0);
+		if (!is_addr_unspecified(&flags->parsed_media_address))
+			len = sprintf(buf, "%s %s",
+					flags->parsed_media_address.family->rfc_name,
+					sockaddr_print_buf(&flags->parsed_media_address));
+		else
+			call_stream_address46(buf, ps, SAF_NG, &len, NULL, 0);
 		chopper_append_printf(chop, " IN %.*s", len, buf);
 	}
 	chopper_append_c(chop, "\r\n");
 }
 
 
+static void sdp_version_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologue *monologue) {
+	char version_str[64];
+	snprintf(version_str, sizeof(version_str), "%llu", monologue->sdp_version);
+	size_t version_len = strlen(version_str);
+	chop->offset = 0; // start from the top
+
+	for (GList *l = sessions->head; l; l = l->next) {
+		struct sdp_session *session = l->data;
+		struct sdp_origin *origin = &session->origin;
+		// update string unconditionally to keep position tracking intact
+		chopper_replace(chop, &origin->version_str, &origin->version_output_pos, version_str, version_len);
+	}
+}
+
+static void sdp_version_check(struct sdp_chopper *chop, GQueue *sessions, struct call_monologue *monologue) {
+	// we really expect only a single session here, but we treat all the same regardless,
+	// and use the same version number on all of them
+
+	// first update all versions to match our single version
+	sdp_version_replace(chop, sessions, monologue);
+	// then check if we need to change
+	if (!monologue->last_sdp)
+		goto dup;
+	if (g_string_equal(monologue->last_sdp, chop->output))
+		return;
+
+	// mismatch detected. increment version, update again, and store copy
+	monologue->sdp_version++;
+	sdp_version_replace(chop, sessions, monologue);
+	g_string_free(monologue->last_sdp, TRUE);
+dup:
+	monologue->last_sdp = g_string_new_len(chop->output->str, chop->output->len);
+}
+
+
 /* called with call->master_lock held in W */
 int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologue *monologue,
 		struct sdp_ng_flags *flags)
@@ -2285,21 +2452,60 @@ int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologu
 	int media_index, sess_conn;
 	struct call_media *call_media;
 	struct packet_stream *ps, *ps_rtcp;
+	const char *err = NULL;
 
 	m = monologue->medias.head;
+	media_index = 1;
 
 	for (l = sessions->head; l; l = l->next) {
 		session = l->data;
+		err = "no matching session media";
 		if (!m)
 			goto error;
 		call_media = m->data;
-		if (call_media->index != 1)
+		err = "mismatched session media index";
+		if (call_media->index != media_index)
 			goto error;
+		err = "no matching session media stream";
 		j = call_media->streams.head;
 		if (!j)
 			goto error;
 		ps = j->data;
 
+		err = "error while processing o= line";
+		if (!monologue->sdp_username)
+			monologue->sdp_username = call_strdup_len(monologue->call, session->origin.username.s,
+					session->origin.username.len);
+		else if (flags->replace_username) {
+			if (copy_up_to(chop, &session->origin.username))
+				goto error;
+			chopper_append_c(chop, monologue->sdp_username);
+			if (skip_over(chop, &session->origin.username))
+				goto error;
+		}
+
+		// record position of o= line and init SDP version
+		if (copy_up_to(chop, &session->origin.version_str))
+			goto error;
+		session->origin.version_output_pos = chop->output->len;
+		if (!monologue->sdp_version) {
+			monologue->sdp_version = session->origin.version_num;
+			if (monologue->sdp_version == 0 || monologue->sdp_version == ULLONG_MAX)
+				monologue->sdp_version = ssl_random();
+		}
+
+		err = "error while processing s= line";
+		if (!monologue->sdp_session_name)
+			monologue->sdp_session_name = call_strdup_len(monologue->call, session->session_name.s,
+					session->session_name.len);
+		else if (flags->replace_sess_name) {
+			if (copy_up_to(chop, &session->session_name))
+				goto error;
+			chopper_append_c(chop, monologue->sdp_session_name);
+			if (skip_over(chop, &session->session_name))
+				goto error;
+		}
+
 		sess_conn = 0;
 		if (flags->replace_sess_conn)
 			sess_conn = 1;
@@ -2314,17 +2520,20 @@ int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologu
 		}
 
 		if (session->origin.parsed && flags->replace_origin &&
-		    !flags->ice_force_relay) {
+		    flags->ice_option != ICE_FORCE_RELAY) {
+			err = "failed to replace network address";
 			if (replace_network_address(chop, &session->origin.address, ps, flags, 0))
 				goto error;
 		}
 		if (session->connection.parsed && sess_conn &&
-		    !flags->ice_force_relay) {
+		    flags->ice_option != ICE_FORCE_RELAY) {
+			err = "failed to replace network address";
 			if (replace_network_address(chop, &session->connection.address, ps, flags, 1))
 				goto error;
 		}
 
 		if (!MEDIA_ISSET(call_media, PASSTHRU)) {
+			err = "failed to process session attributes";
 			if (process_session_attributes(chop, &session->attributes, flags))
 				goto error;
 		}
@@ -2337,39 +2546,51 @@ int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologu
 			chopper_append_c(chop, "\r\n");
 		}
 
-		media_index = 1;
+		if (MEDIA_ISSET(call_media, ICE) && MEDIA_ISSET(call_media, ICE_LITE_SELF))
+			chopper_append_c(chop, "a=ice-lite\r\n");
+
 
 		for (k = session->media_streams.head; k; k = k->next) {
 			sdp_media = k->data;
+			err = "no matching media";
 			if (!m)
 				goto error;
 			call_media = m->data;
+			err = "media index mismatched";
 			if (call_media->index != media_index)
 				goto error;
+			err = "no matching media stream";
 			j = call_media->streams.head;
 			if (!j)
 				goto error;
 			ps = j->data;
 
-			if (!flags->ice_force_relay && call_media->type_id != MT_MESSAGE) {
+			if (flags->ice_option != ICE_FORCE_RELAY && call_media->type_id != MT_MESSAGE) {
+				err = "failed to replace media type";
 				if (replace_media_type(chop, sdp_media, call_media))
 					goto error;
+				err = "failed to replace media port";
 			        if (replace_media_port(chop, sdp_media, ps))
 				        goto error;
+				err = "failed to replace media port count";
 			        if (replace_consecutive_port_count(chop, sdp_media, ps, j))
 				        goto error;
+				err = "failed to replace media protocol";
 				if (replace_transport_protocol(chop, sdp_media, call_media))
 				        goto error;
+				err = "failed to replace media formats";
 				if (replace_codec_list(chop, sdp_media, call_media))
 					goto error;
 
 				if (sdp_media->connection.parsed) {
+					err = "failed to replace media network address";
 				        if (replace_network_address(chop, &sdp_media->connection.address, ps,
 								flags, 1))
 					        goto error;
 				}
 			}
 			else if (call_media->type_id == MT_MESSAGE) {
+				err = "failed to generate connection line";
 				if (!sdp_media->connection.parsed)
 					if (synth_session_connection(chop, sdp_media))
 						goto error;
@@ -2377,6 +2598,7 @@ int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologu
 				goto next;
 			}
 
+			err = "failed to process media attributes";
 			if (process_media_attributes(chop, sdp_media, flags, call_media))
 				goto error;
 
@@ -2400,6 +2622,7 @@ int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologu
 			if (ps->rtcp_sibling) {
 				ps_rtcp = ps->rtcp_sibling;
 				j = j->next;
+				err = "no RTCP sibling";
 				if (!j)
 					goto error;
 				assert(j->data == ps_rtcp);
@@ -2426,7 +2649,7 @@ int sdp_replace(struct sdp_chopper *chop, GQueue *sessions, struct call_monologu
 					chopper_append_c(chop, "a=rtcp-mux\r\n");
 					ps_rtcp = NULL;
 				}
-				else if (ps_rtcp && !flags->ice_force_relay) {
+				else if (ps_rtcp && flags->ice_option != ICE_FORCE_RELAY) {
 					insert_rtcp_attr(chop, ps_rtcp, flags);
 					if (MEDIA_ISSET(call_media, RTCP_MUX))
 						chopper_append_c(chop, "a=rtcp-mux\r\n");
@@ -2466,10 +2689,15 @@ next:
 	}
 
 	copy_remainder(chop);
+
+	if (flags->replace_sdp_version)
+		sdp_version_check(chop, sessions, monologue);
+
+
 	return 0;
 
 error:
-	ilog(LOG_ERROR, "Error rewriting SDP");
+	ilog(LOG_ERROR, "Error rewriting SDP: %s", err);
 	return -1;
 }
 
