@@ -19,16 +19,21 @@ struct rtp_header {
 struct rtp_payload_type {
 	int payload_type;
 	str encoding_with_params; // "opus/48000/2"
+	str encoding_with_full_params; // "opus/48000/1"
 	str encoding; // "opus"
 	unsigned int clock_rate; // 48000
 	str encoding_parameters; // "2"
 	int channels; // 2
 	str format_parameters; // value of a=fmtp
+	str codec_opts; // extra codec-specific options
+	GQueue rtcp_fb; // a=rtcp-fb:...
 
 	int ptime; // default from RFC
 	int bitrate;
 
 	const codec_def_t *codec_def;
+
+	unsigned int for_transcoding:1;
 };
 
 
